<?php
/**
 * Admin scripts used for WC data editing.
 * @author Payment Plugins
 * @copyright 2016 Payment Plugins
 *
 */
class Braintree_Gateway_Admin_Assets
{

	public static function init()
	{
		add_action( 'admin_enqueue_scripts', __CLASS__ . '::enqueue_scripts' );
	}

	public static function enqueue_scripts()
	{
		$screen = get_current_screen();
		$screen_id = $screen ? $screen->id : '';
		$js_admin = bt_manager()->plugin_assets_path() . 'js/admin/';
		$js_path = bt_manager()->plugin_assets_path() . 'js/';
		$css_admin = bt_manager()->plugin_assets_path() . 'css/admin/';
		$css_path = bt_manager()->plugin_assets_path() . 'css/';
		$version = bt_manager()->version;
		$page = preg_match( '/braintree-gateway.*?_page_(.*)/', $screen_id, $matches ) ? ( isset( $matches [ 1 ] ) ? $matches [ 1 ] : $screen_id ) : $screen_id;
		
		$plugin_activation = ! empty( $_GET [ 'plugin-activation' ] );
		
		wp_register_script( 'braintree-admin-materialize-js', $js_admin . 'materialize.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-settings-js', $js_admin . 'admin-settings.js', array (
				'braintree-admin-materialize-js', 
				'bfwc-select2' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-notices', $js_admin . 'admin-notices.js', array (
				'braintree-admin-materialize-js' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-donation-settings-js', $js_admin . 'donation-settings.js', array (
				'braintree-admin-settings-js' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-edit-donation-js', $js_admin . 'edit-donation.js', array (
				'braintree-admin-materialize-js', 
				'bfwc-select2' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-logs-js', $js_admin . 'logs.js', array (
				'braintree-admin-materialize-js' 
		), $version, true );
		
		wp_register_script( 'braintree-wc-order-js', $js_admin . 'order.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'braintree-wc-product-data-js', $js_admin . 'product-data.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'braintree-datatables-js', $js_admin . 'jquery/dataTables.min.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'braintree-datatables-buttons-js', $js_admin . 'jquery/dataTables.buttons.min.js', array (
				'braintree-datatables-js' 
		), $version, true );
		
		wp_register_script( 'braintree-buttons-flash-js', $js_admin . 'jquery/buttons.flash.min.js', array (
				'braintree-datatables-buttons-js' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-color-picker-colors-js', $js_admin . 'tinyColorPicker-master/colors.js', array (
				'braintree-admin-donation-settings-js' 
		), $version, true );
		
		wp_register_script( 'braintree-admin-color-picker-script', $js_admin . 'tinyColorPicker-master/jqColorPicker.js', array (
				'braintree-admin-color-picker-colors-js' 
		), $version, true );
		wp_register_script( 'braintree-admin-hooks-js', $js_admin . 'webhook-test.js', array (
				'jquery' 
		), $version, true );
		wp_register_script( 'braintree-admin-spin-js', $js_admin . 'spin.min.js', array (
				'jquery' 
		), $version, true );
		wp_register_script( 'braintree-wc-subscription-metabox-js', $js_admin . 'subscription-metabox.js', array (
				'jquery' 
		), $version, true );
		wp_register_script( 'braintree-admin-users', $js_admin . 'users.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'bfwc-admin-messages', $js_admin . 'admin-messages.js', array (
				'jquery', 
				'bfwc-select2', 
				'braintree-admin-settings-js' 
		), $version, true );
		
		wp_register_script( 'bfwc-select2', $js_path . 'select2/select2.min.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_script( 'bfwc-help-widget', $js_admin . 'help-widget.js', array (
				'jquery' 
		), $version, true );
		
		wp_register_style( 'braintree-admin-materialize-css', $css_admin . 'materialize.css', null, $version, null );
		wp_register_style( 'braintree-admin-product-css', $css_admin . 'product-data.css', null, $version, null );
		wp_register_style( 'braintree-admin-datatables-css', $css_admin . 'jquery/dataTables.css', null, $version, null );
		wp_register_style( 'braintree-admin-materialize-graphics', 'https://fonts.googleapis.com/icon?family=Material+Icons', null, $version, null );
		wp_register_style( 'braintree-admin-datatables-buttons-css', $css_admin . 'jquery/dataTables.buttons.min.css', null, $version, null );
		wp_register_style( 'bfwc-admin-subscriptions-css', $css_admin . 'subscriptions.css', null, $version, null );
		wp_register_style( 'bfwc-select2-css', $css_path . 'select2.css', null, $version, null );
		
		$wc_product_screens = array (
				'product', 
				'edit-product' 
		);
		$wc_order_screens = array (
				'shop_order', 
				'bfwc_subscription', 
				'edit-bfwc_subscription' 
		);
		$donation_screens = array (
				'braintree_donation', 
				'bt_rc_donation' 
		);
		
		if ( in_array( $screen_id, $wc_product_screens ) ) {
			
			wp_enqueue_style( 'braintree-admin-product-css' );
			wp_enqueue_script( 'braintree-wc-product-data-js' );
			wp_localize_script( 'braintree-wc-product-data-js', 'braintree_product_data', self::get_product_data() );
		}
		if ( in_array( $screen_id, $wc_order_screens ) ) {
			wp_enqueue_script( 'braintree-wc-order-js' );
			wp_enqueue_style( 'bfwc-admin-subscriptions-css' );
			
			if ( $screen_id === 'bfwc_subscription' ) {
				wp_enqueue_script( 'braintree-wc-subscription-metabox-js' );
				wp_localize_script( 'braintree-wc-subscription-metabox-js', 'bfwc_subscription_vars', Braintree_Gateway_Admin_Subscription_Metabox::localize_data() );
			}
		}
		if ( in_array( $screen_id, $donation_screens ) ) {
			wp_enqueue_script( 'braintree-admin-edit-donation-js' );
			wp_enqueue_script( 'braintree-admin-notices' );
			wp_enqueue_script( 'braintree-datatables-js' );
			wp_enqueue_style( 'braintree-admin-materialize-css' );
			wp_enqueue_style( 'braintree-admin-materialize-graphics' );
			wp_enqueue_style( 'bfwc-select2-css' );
		}
		
		$all_pages = bfwc_admin_pages();
		
		if ( in_array( $page, $all_pages ) ) {
			wp_enqueue_script( 'braintree-admin-settings-js' );
			wp_enqueue_script( 'braintree-admin-notices' );
			wp_enqueue_script( 'braintree-admin-donation-settings-js' );
			wp_enqueue_script( 'braintree-admin-color-picker-colors-js' );
			wp_enqueue_script( 'braintree-admin-color-picker-script' );
			wp_enqueue_script( 'bfwc-select2' );
			wp_enqueue_script( 'bfwc-help-widget' );
			wp_localize_script( 'braintree-admin-settings-js', 'braintree_settings_vars', apply_filters( 'braintree_settings_localized_variables', array () ) );
			
			wp_enqueue_style( 'braintree-admin-materialize-css' );
			wp_enqueue_style( 'braintree-admin-materialize-graphics' );
			wp_enqueue_style( 'bfwc-select2-css' );
			
			if ( preg_match( '/^braintree.*logs/', $page ) ) {
				wp_enqueue_script( 'braintree-admin-logs-js' );
				wp_enqueue_script( 'braintree-buttons-flash-js' );
				wp_localize_script( 'braintree-admin-logs-js', 'braintree_log_vars', self::log_vars() );
			}
			if ( $page === 'braintree-webhook-test' ) {
				wp_enqueue_script( 'braintree-admin-hooks-js' );
				wp_enqueue_script( 'braintree-admin-spin-js' );
				wp_localize_script( 'braintree-admin-hooks-js', 'braintree_webhook_vars', self::webhook_vars() );
			}
			if ( $page === 'braintree-messages-page' ) {
				wp_enqueue_script( 'bfwc-admin-messages' );
				wp_localize_script( 'bfwc-admin-messages', 'bfwc_admin_messages', Braintree_Gateway_Admin_Messages::localize_messages() );
			}
			
			if ( $plugin_activation ) {
				wp_enqueue_script( 'braintree-plugin-activation', $js_admin . 'plugin-activation.js', array (
						'jquery', 
						'braintree-admin-materialize-js' 
				), $version, true );
				wp_localize_script( 'braintree-plugin-activation', 'bfwc_admin_plugin_activated', self::localize_plugin_activation() );
			}
		
		}
	
	}

	public static function get_product_data()
	{
		global $post;
		
		$variations = get_posts( array (
				'post_type' => 'product_variation', 
				'post_status' => array (
						'private', 
						'publish' 
				), 
				'posts_per_page' => - 1, 
				'orderby' => array (
						'menu_order' => 'ASC', 
						'ID' => 'DESC' 
				), 
				'post_parent' => $post->ID 
		) );
		
		$data = array (
				'posts' => array (
						$post->ID => array (
								'frequency' => get_post_meta( $post->ID, '_subscription_period_interval', true ) 
						) 
				), 
				'enabled' => array (
						'wc_subscriptions_active' => bt_manager()->is_woocommerce_subscriptions_active() 
				), 
				'product_type' => 'braintree-subscription', 
				'products' => array (
						'braintree-subscription' => array (
								'html' => '<li class="product-plan"><a href="#" class="select2"></a><span>%desc</span><input type="hidden" name="_braintree_%env_plans[%curr]" id="" value="%value"></li>' 
						), 
						'braintree-variable-subscription' => array (
								'html' => '<li class="product-plan"><a href="#" class="select2"></a><span>%desc</span><input type="hidden" name="variable_braintree_%env_plans[%loop][%curr]" id="" value="%value"></li>' 
						), 
						'subscription' => array (
								'html' => '<li class="product-plan"><a href="#" class="select2"></a><span>%desc</span><input type="hidden" name="_braintree_%env_plans[%curr]" id="" value="%value"></li>' 
						), 
						'variable-subscription' => array (
								'html' => '<li class="product-plan"><a href="#" class="select2"></a><span>%desc</span><input type="hidden" name="variable_braintree_%env_plans[%loop][%curr]" id="" value="%value"></li>' 
						) 
				), 
				'environments' => array (
						'sandbox' => array (
								'plans' => get_transient( 'braintree_wc_sandbox_plans' ), 
								'saved_plans' => get_post_meta( $post->ID, 'braintree_sandbox_plans', true ) 
						), 
						'production' => array (
								'active' => false, 
								'plans' => get_transient( 'braintree_wc_production_plans' ), 
								'saved_plans' => get_post_meta( $post->ID, 'braintree_production_plans', true ) 
						) 
				), 
				'messages' => array (
						'duplicate' => __( 'Plan %s has already been assigned with currency %c. If you want to change the plan for currency %c please remove plan %s', 'braintree' ), 
						'inactive_license' => __( 'You must purchase a license before adding Production Braintree Plans', 'braintree' ), 
						'invalid_frequency' => __( 'Plan %s cannot be added. Plans associated with a subscription must have the same billing frequency.', 'braintree' ) 
				), 
				'trial_period_singular' => bfwc_billing_periods_string( 'singular' ), 
				'trial_period_plural' => bfwc_billing_periods_string( 'plural' ) 
		);
		if ( $variations ) {
			foreach ( $variations as $variation ) {
				$data [ 'posts' ] [ $variation->ID ] = array (
						'frequency' => get_post_meta( $variation->ID, '_subscription_period_interval', true ) 
				);
			}
		}
		
		return $data;
	}

	public static function log_vars()
	{
		$vars = array (
				'search_placeholder' => __( 'Enter Search Criteria', 'braintree' ) 
		);
		return $vars;
	}

	public static function webhook_vars()
	{
		$vars = array (
				'ajax_url' => admin_url() . 'admin-ajax.php?action=bfwc_admin_retrieve_payload', 
				'webhook_url' => get_rest_url( null, bt_manager()->api->get_path() ) 
		);
		return $vars;
	}

	public static function localize_plugin_activation()
	{
		$data = array ();
		ob_start();
		include 'views/activation-modal.php';
		$data [ 'html' ] = ob_get_clean();
		return $data;
	}

}
Braintree_Gateway_Admin_Assets::init();