<?php
class Braintree_Gateway_Admin_Subscription_Metabox
{
	
	private static $saving_order_items = false;

	public static function init()
	{
		add_action( 'current_screen', __CLASS__ . '::maybe_update_subscription_statuses' );
		add_action( 'add_meta_boxes', __CLASS__ . '::add_meta_boxes' );
		
		// use when saving existing subscription data.
		add_action( 'woocommerce_process_shop_order_meta', __CLASS__ . '::save_subscription_data', 999, 2 );
	}

	public static function maybe_update_subscription_statuses( $screen )
	{
		$screen_id = $screen ? $screen->id : '';
		
		if ( $screen_id === 'bfwc_subscription' ) {
			add_filter( 'wc_order_statuses', function ( $order_statuses ) {
				$subscription_statuses = bfwc_get_subscription_statuses();
				$subscription_statuses[ 'wc-pending' ] = $order_statuses[ 'wc-pending' ];
				return $subscription_statuses;
			} );
		}
	}

	public static function add_meta_boxes()
	{
		add_meta_box( 'bfwc-subscriptions-metabox', __( 'Related Orders', 'braintree' ), __CLASS__ . '::output_related_orders', 'bfwc_subscription', 'normal', 'low' );
		add_meta_box( 'bfwc-subscription-options-metabox', __( 'Subscription Options', 'braintree' ), __CLASS__ . '::output_subscription_options', 'bfwc_subscription', 'side', 'default' );
	}

	/**
	 *
	 * @param WP_Post $post 
	 */
	public static function output_related_orders( $post )
	{
		$subscription = bfwcs_get_subscription( $post->ID );
		$orders = bfwcs_get_related_orders( $subscription );
		if ( $subscription->order ) {
			$orders[] = $subscription->order;
		}
		if ( $orders ) {
			bfwc_admin_get_template( 'meta-box-html/related-orders.php', array ( 
					'orders' => $orders, 
					'subscription' => $subscription 
			) );
		}
	}

	/**
	 *
	 * @param WP_Post $post 
	 */
	public static function output_subscription_options( $post )
	{
		$plans = bfwc_admin_get_subscription_plans();
		$subscription = bfwcs_get_subscription( $post->ID );
		bfwc_admin_get_template( 'meta-box-html/subscription-metabox-options.php', array ( 
				'subscription_plans' => $plans, 
				'subscription' => $subscription 
		) );
	}

	/**
	 * Save the subscription meta data.
	 *
	 * @param int $post_id 
	 * @param WP_Post $post 
	 */
	public static function save_subscription_data( $post_id, $post )
	{
		if ( ! isset( $_POST[ '_bfwc_subscription' ] ) || ! wp_verify_nonce( $_POST[ '_bfwc_subscription' ], 'bfwc-subscription' ) ) {
			return;
		}
		
		$subscription = bfwcs_get_subscription( $post_id );
		
		$has_trial = ! empty( $_POST[ '_subscription_trial_end_date' ] );
		
		// only update the subscription dates if the subscription hasn't been created in Braintree.
		if ( ! $subscription->is_created() ) {
			// there is a trial so calculate the dates.
			if ( $has_trial ) {
				$trial_end_date = sprintf( '%s 00:00:00', wc_clean( $_POST[ '_subscription_trial_end_date' ] ) );
				$trial_end_date = DateTime::createFromFormat( 'Y-m-d H:i:s', $trial_end_date, new DateTimeZone( bfwc_get_gateway_timezone() ) );
				$trial_end_date->setTimezone( new DateTimeZone( 'UTC' ) );
				$subscription->update_date( 'trial_end', $trial_end_date );
				$subscription->update_date( 'start', $trial_end_date );
				$subscription->update_date( 'next_payment', $trial_end_date );
			} else {
				$start_date = sprintf( '%s 00:00:00', wc_clean( $_POST[ '_start_date' ] ) );
				$start_date = DateTime::createFromFormat( 'Y-m-d H:i:s', $start_date, new DateTimeZone( bfwc_get_gateway_timezone() ) );
				$start_date->setTimezone( new DateTimeZone( 'UTC' ) );
				$subscription->update_date( 'start', $start_date );
				$subscription->update_date( 'next_payment', $start_date );
				$subscription->update_date( 'trial_end', 0 );
			}
			update_post_meta( $post_id, '_subscription_length', wc_clean( $_POST[ '_subscription_length' ] ) );
			
			$braintree_plan = wc_clean( $_POST[ '_subscription_plan' ] );
			$plans = bfwc_admin_get_subscription_plans();
			$plan = $plans[ $braintree_plan ];
			
			update_post_meta( $post_id, '_braintree_plan', $braintree_plan );
			update_post_meta( $post_id, '_merchant_account_id', bwc_get_merchant_account( $plan[ 'currencyIsoCode' ] ) );
			update_post_meta( $post_id, '_subscription_time_zone', bfwc_get_gateway_timezone() );
			update_post_meta( $post_id, '_subscription_period_interval', $plan[ 'billingFrequency' ] );
			update_post_meta( $post_id, '_order_currency', $plan[ 'currencyIsoCode' ] );
			update_post_meta( $post_id, '_subscription_period', 'month' );
			
			// calculate the end date.
			switch ($subscription->subscription_period) {
				case 'month' :
					$date = $subscription->get_date( 'next_payment' );
					$date->add( new DateInterval( 'P' . $subscription->subscription_length . 'M' ) );
					$subscription->update_date( 'end', $date );
					break;
			}
		}
	}

	public static function localize_data()
	{
		// create localized text for frequency. every month, every 2 months etc
		
		return apply_filters( 'bfwc_subscription_localized_data', array ( 
				'plans' => bfwc_admin_get_subscription_plans(), 
				'billing_frequency_text' => array ( 
						'intervals' => bfwcs_billing_interval_string() 
				), 
				'trial_text' => array ( 
						'singluar' => bfwc_billing_periods_string( 'singular' ), 
						'plural' => bfwc_billing_periods_string( 'plural' ) 
				) 
		) );
	}
}
Braintree_Gateway_Admin_Subscription_Metabox::init();