<?php

class Braintree_Gateway_Custom_Form_Settings extends Braintree_Gateway_Settings_API
{

	public function __construct()
	{
		$this->tab = 'checkout-settings';
		$this->page = 'braintree-gateway-settings';
		$this->id = 'checkout_settings';
		$this->title = array (
				'title' => __( 'Custom Form Settings', 'braintree' ), 
				'description' => __( '', 'braintree' ) 
		);
		add_filter( 'braintree_gateway_default_settings', array (
				$this, 
				'get_default_settings' 
		) );
		add_action( "braintree_gateway_{$this->tab}_save_settings", array (
				$this, 
				'save' 
		) );
		add_filter( 'braintree_settings_localized_variables', array (
				$this, 
				'localize_vars' 
		) );
		$this->add_validate_filters();
	}

	public function localize_vars( $vars )
	{
		$vars [ 'custom_forms' ] = array ();
		$forms = bwc_get_custom_forms();
		foreach ( $this->settings() [ 'custom_form_design' ] [ 'options' ] as $form => $value ) {
			$vars [ 'custom_forms' ] [ $form ] = array (
					'styles' => $forms [ $form ] [ 'default_styles' ] 
			);
		}
		return $vars;
	}

	public function add_validate_filters()
	{
		$validate = array (
				'custom_form_styles' => 'validate_json', 
				'custom_form_loader_css' => 'validate_json' 
		);
		foreach ( $validate as $k => $method ) {
			add_filter( "braintree_gateway_validate_{$k}", array (
					$this, 
					$method 
			), 10, 2 );
		}
	}

	public function settings()
	{
		return array (
				'custom_form_design' => array (
						'type' => 'select', 
						'title' => __( 'Custom Form', 'braintree' ), 
						'default' => 'bootstrap_form', 
						'options' => $this->get_form_options(), 
						'value' => '', 
						'class' => '', 
						'link' => array (
								'url' => 'https://support.paymentplugins.com/hc/en-us/articles/115001527088-Custom-Forms', 
								'text' => __( 'View live examples', 'braintree' ) 
						), 
						'tool_tip' => true, 
						'description' => __( 'This option allows you to select the payment form which will be displayed on the checkout page.', 'braintree' ) 
				), 
				'custom_form_styles' => array (
						'type' => 'textarea', 
						'title' => __( 'Styles', 'braintree' ), 
						'default' => '{"input":{"font-size":"14px","font-family":"helvetica, tahoma, calibri, sans-serif","color":"#3a3a3a"}}', 
						'placeholder' => '{"input":{  "font-size":"14px","font-family":"helvetica, tahoma, calibri, sans-serif","color":"#3a3a3a"}}', 
						'class' => array (), 
						'tool_tip' => true, 
						'description' => __( 'You can customize the css of the hosted payment fields using this setting. All css must be in json format or it will cause errors. If the styles are left blank, the default styles will be applied automatically. Please reference https://developers.braintreepayments.com/guides/hosted-fields/styling/javascript/v3 if you have questions on how to format the custom css.', 'braintree' ) 
				), 
				'dynamic_card_display' => array (
						'type' => 'checkbox', 
						'title' => __( 'Dynamic Card display' ), 
						'default' => 'yes', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, the payment form will display the card type dynamically as the customer enters their payment information.' ), 
						'helper' => array (
								'enabled' => true, 
								'title' => __( 'Dynamic Card Display', 'braintree' ), 
								'type' => 'img', 
								'description' => __( 'If enabled, an icon of the card type being entered by the customer will appear. This provides dynamic feedback to the customer as they enter their information.', 'braintree' ), 
								'url' => 'https://wordpress.paymentplugins.com/woo-payment-gateway/assets/dynamic_card_display.png' 
						) 
				), 
				'postal_field_enabled' => array (
						'type' => 'checkbox', 
						'title' => __( 'Display Postal Field', 'braintree' ), 
						'default' => 'yes', 
						'value' => 'yes', 
						'class' => 'filled-in', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, the custom form that you select will display a field for the customer to enter the postal code associated with their payment method.', 'braintree' ) 
				), 
				'cvv_field_enabled' => array (
						'type' => 'checkbox', 
						'title' => __( 'Display CVV Field', 'braintree' ), 
						'default' => 'yes', 
						'value' => 'yes', 
						'class' => 'filled-in', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, the custom form that you select will display a field for the customer to enter their card security code (CVV).', 'braintree' ) 
				), 
				'card_number_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'Number Placeholder', 'braintree' ), 
						'default' => __( 'Card Number', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value entered here will display as a placeholder on the credit card number field.', 'braintree' ) 
				), 
				'card_cvv_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'CVV Placeholder', 'braintree' ), 
						'default' => __( 'CVV', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value entered here will display as a placeholder on the credit card security code field.', 'braintree' ) 
				), 
				'card_expiration_date_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'Expiration Date Placeholder', 'braintree' ), 
						'default' => __( 'MM/YY', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'For forms that contain a combined expiration date and year, this will be the placeholder.', 'braintree' ) 
				), 
				'card_expiration_month_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'Expiration Month Placeholder', 'braintree' ), 
						'default' => __( 'MM', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value entered here will display as a placeholder on the expiration month.', 'braintree' ) 
				), 
				'card_expiration_year_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'Expiration Year Placeholder', 'braintree' ), 
						'default' => __( 'YY', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value entered here will display as a placeholder on the expiration year.', 'braintree' ) 
				), 
				'card_postal_placeholder' => array (
						'type' => 'text', 
						'title' => __( 'Card Postal Placeholder', 'braintree' ), 
						'default' => __( 'Postal Code', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value entered here will display as a placeholder on the credit card postal code field.', 'braintree' ) 
				), 
				'card_number_label' => array (
						'type' => 'text', 
						'title' => __( 'Card Number Label', 'braintree' ), 
						'default' => __( 'Card Number', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card number input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_cvv_label' => array (
						'type' => 'text', 
						'title' => __( 'CVV Label', 'braintree' ), 
						'default' => __( 'CVV', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card cvv input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_postal_label' => array (
						'type' => 'text', 
						'title' => __( 'Postal Label', 'braintree' ), 
						'default' => __( 'Postal Code', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card postal input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_expiration_date_label' => array (
						'type' => 'text', 
						'title' => __( 'Exp Date Label', 'braintree' ), 
						'default' => __( 'Exp Date', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card expiration date input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_expiration_month_label' => array (
						'type' => 'text', 
						'title' => __( 'Exp Month Label', 'braintree' ), 
						'default' => __( 'Month', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card expiration month input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_expiration_year_label' => array (
						'type' => 'text', 
						'title' => __( 'Exp Year Label', 'braintree' ), 
						'default' => __( 'Year', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card expiration year input field for forms that contain a label.', 'braintree' ) 
				), 
				'card_save_label' => array (
						'type' => 'text', 
						'title' => __( 'Save Card Label', 'braintree' ), 
						'default' => __( 'Save', 'braintree' ), 
						'value' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This text will display above or next to the card save field for forms that contain a label.', 'braintree' ) 
				), 
				'enable_loader' => array (
						'type' => 'checkbox', 
						'title' => __( 'Checkout Loader', 'braintree' ), 
						'default' => '', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'class' => 'filled-in', 
						'description' => __( 'If enabled, a loader will appear around the payment form to let the customer know that the payment is processing.', 'braintree' ) 
				), 
				'custom_form_loader_message' => array (
						'type' => 'textarea', 
						'title' => __( 'Loader HTML', 'braintree' ), 
						'default' => __( '<div class="custom-form-checkout-loader"><h1>Processing...</h1></div>', 'braintree' ), 
						'value' => '', 
						'tool_tip' => true, 
						'description' => __( 'This is the html that will appear inside the loader overaly. You can add your own animation here or custom html. The customizations are endless.', 'braintree' ) 
				), 
				'custom_form_loader_css' => array (
						'type' => 'textarea', 
						'title' => __( 'Loader CSS', 'braintree' ), 
						'default' => '{"border":"none","width":"80%","max-width":"300px"}', 
						'placeholder' => '', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This option allows you to enter custom css for the loader. The css must be entered in json format.' ) 
				) 
		);
	}

	public function output_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'html-helpers/settings-modal.php';
	}

	public function get_form_options()
	{
		$forms = bwc_get_custom_forms();
		$form_options = array ();
		foreach ( $forms as $k => $data ) {
			$form_options [ $k ] = $data [ 'label' ];
		}
		return $form_options;
	}
}