<?php
/**
 * Simple subscription product that is used to represent a Braintree Subscription.
 * @author Payment Plugins
 * @copyright 2016 Payment Plugins
 *
 *@property string $subscription_price price of the subscription.
 *@property string $subscription_period billing period.
 *@property string $subscription_period_interval billing internval. ex monthly, every 2nd month.
 *@property string $subscription_length length of the subscription.
 *@property string $subscription_trial_length length of the trial period.
 *@property string $subscription_trial_period trial period. ex day, month.
 *@property string $subscription_sign_up_fee signup fee associated with the subscription.
 */
class WC_Product_Braintree_Subscription extends WC_Product_Simple
{

	public function __construct( $product )
	{
		parent::__construct( $product );
		
		$this->subscription_period = 'month';
		$this->product_type = 'braintree-subscription';
	}

	public function get_price_html( $price = '' )
	{
		return bfwcs_get_product_price_html( $this, $price );
	}

	public function get_plans()
	{
		$var = 'braintree_' . bt_manager()->get_environment() . '_plans';
		return $this->$var;
	}

	/**
	 * Return the subscription signup fee.
	 *
	 * @return string $fee;
	 */
	public function get_signup_fee()
	{
		return $this->subscription_sign_up_fee ? $this->subscription_sign_up_fee : 0;
	}

	/**
	 * Return true if shipping is only charged once for the product.
	 * If there is a trial period then shipping must be included
	 * in all charges if there is any shipping.
	 */
	public function is_one_time_shipping()
	{
		return $this->subscription_one_time_shipping === 'yes' && ! $this->subscription_trial_length;
	}
}