<?php
use Braintree\PaymentMethod;
if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
	return;
}

/**
 * Gateway class that processess WooCommrce orders.
 *
 * @author Payment Plugins
 * @copyright 2016 Payment Plugins
 *           
 */
class WC_Braintree_Payment_Gateway extends WC_Payment_Gateway
{
	const ID = 'braintree_payment_gateway';
	
	/**
	 * Nonce id contained in the $_POST.
	 *
	 * @var string
	 */
	public static $nonce_id = 'braintree_payment_gateway_nonce';
	
	/**
	 * Device data field key located in $_POST.
	 *
	 * @var string
	 */
	public static $device_data_id = 'braintree_device_data';
	
	/**
	 * The key used in $_POST to identify the selected payment method token.
	 *
	 * @var string
	 */
	public static $token_id = 'braintree_payment_gateway_method_token';
	
	/**
	 * Nonce that represents a payment method.
	 *
	 * @var String
	 */
	public $nonce = '';
	
	/**
	 * Token that represents a vaulted payment method.
	 *
	 * @var string
	 */
	public static $payment_method_token = '';
	
	/**
	 * Name of the save payment method checkbox.
	 *
	 * @var string
	 */
	public static $save_method_name = 'bfwc_save_credit_card';
	
	/**
	 *
	 * @var WC_Order
	 */
	public $order;

	public function __construct()
	{
		$this->enabled = bt_manager()->get_option( 'enabled' ) && ! bwc_is_paypal_only() ? 'yes' : 'no';
		
		$this->id = self::ID;
		
		$this->title = bt_manager()->get_option( 'title_text' );
		
		$this->method_title = __( 'Braintree Payment Gateway', 'braintree' );
		
		$this->has_fields = true;
		
		$this->actions();
		
		$this->set_supports();
	}

	public function actions()
	{
		// Only add actions and filters if the gateway is available.
		if ( $this->is_available() ) {
			add_filter( "braintree_wc_{$this->id}_process_order", array (
					$this, 
					'process_order' 
			) );
			add_filter( "braintree_wc_{$this->id}_process_subscription", array (
					$this, 
					'process_subscription' 
			) );
			add_filter( "bfwc_process_{$this->id}_braintree_subscription", array (
					$this, 
					'process_braintree_subscription' 
			) );
			add_action( "braintree_wc_{$this->id}_save_wc_order_meta", array (
					$this, 
					'save_order_meta' 
			), 10, 2 );
			add_filter( 'woocommerce_gateway_icon', array (
					$this, 
					'woocommerce_gateway_icon' 
			), 10, 2 );
			add_filter( 'woocommerce_gateway_title', array (
					$this, 
					'woocommerce_gateway_title' 
			), 10, 2 );
			add_filter( 'woocommerce_saved_payment_methods_list', array (
					$this, 
					'saved_payment_method_list' 
			), 10, 2 );
			add_action( "woocommerce_subscription_payment_method_updated_to_{$this->id}", array (
					$this, 
					'payment_method_updated_to' 
			), 10, 2 );
		}
	}

	public static function init()
	{
		add_filter( 'bwc_add_payment_gateways', __CLASS__ . '::add_braintree_gateway' );
		
		add_filter( 'woocommerce_payment_gateways', __CLASS__ . '::add_gateway' );
		
		add_action( 'wp_loaded', __CLASS__ . '::maybe_delete_payment_method' );
		
		add_filter( 'woocommerce_payment_complete_order_status', __CLASS__ . '::maybe_update_order_status', 99, 2 );
		
		add_action( 'woocommerce_subscription_failing_payment_method_updated_' . static::ID, __CLASS__ . '::update_failing_payment_method', 10, 2 );
		
		// called when a new customer is being inserted into the database.
		add_filter( 'insert_user_meta', __CLASS__ . '::woocommerce_new_customer_data', 10, 3 );
		
		add_action( 'bfwc_before_process_order', __CLASS__ . '::before_order_process', 99 );
		
		add_action( 'bfwc_before_process_order_' . static::ID, __CLASS__ . '::maybe_save_payment_method' );
	
	}

	public static function add_braintree_gateway( $gateways )
	{
		$gateways [] = __CLASS__;
		return $gateways;
	}

	public static function add_gateway( $gateways )
	{
		$gateways [] = __CLASS__;
		return $gateways;
	}

	/**
	 * Process the payment for the WC_Order.
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Payment_Gateway::process_payment()
	 */
	public function process_payment( $order_id )
	{
		$order = wc_get_order( $order_id );
		/**
		 * Perform any functionality needed before processing the order.
		 * If errors, return;
		 */
		do_action( 'bfwc_before_process_order', $order );
		
		if ( wc_notice_count( 'error' ) > 0 ) {
			return $this->order_error();
		}
		
		global $bfwc_order_total;
		
		$bfwc_order_total = $order->get_total();
		
		if ( $this->order_has_subscription( $order_id ) ) {
			
			return apply_filters( "braintree_wc_{$this->id}_process_subscription", $order );
		
		} elseif ( bfwcs_order_contains_subscription( $order_id ) ) {
			
			return apply_filters( "bfwc_process_{$this->id}_braintree_subscription", $order );
		
		} else {
			
			if ( bwcs_is_woocommerce_change_payment() ) {
				return array (
						'result' => 'success', 
						'redirect' => wc_get_page_permalink( 'myaccount' ) 
				);
			}
			
			return apply_filters( "braintree_wc_{$this->id}_process_order", $order );
		}
	}

	/**
	 * Process the WooCommerce order.
	 *
	 * @param WC_Order $order_id        	
	 * @return array
	 */
	public function process_order( $order )
	{
		global $bfwc_order_total;
		$attribs = array (
				'amount' => $order->get_total(), 
				'taxAmount' => $order->get_total_tax() 
		);
		self::add_customer( $attribs, $order );
		self::add_order_payment_method( $attribs );
		self::add_order_id( $attribs, $order );
		self::add_billing_address( $attribs, $order );
		self::add_shipping_address( $attribs, $order );
		self::add_merchant_account( $attribs );
		self::add_partner_code( $attribs );
		self::add_options( $attribs );
		self::add_descriptors( $attribs );
		self::add_device_data( $attribs );
		
		// allow plugins to add additional attributes.
		$attribs = apply_filters( "braintree_woocommerce_{$this->id}_order_attributes", $attribs, $order, $this );
		
		try {
			$result = Braintree_Transaction::sale( $attribs );
			
			if ( $result->success ) {
				do_action( "braintree_wc_{$this->id}_save_wc_order_meta", $order->id, $result->transaction );
				do_action( 'braintree_woocommerce_process_order_success', array (
						'order_id' => $order_id, 
						'result' => $result 
				) );
				
				// ensure the original order total is set as it could be manipulated.
				$order->set_total( $bfwc_order_total );
				
				$order->payment_complete( $result->transaction->id );
				
				WC()->cart->empty_cart();
				return $this->order_success( $order );
			} else {
				wc_add_notice( sprintf( __( 'There was an error processing your payment. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) ), 'error' );
				do_action( 'braintree_woocommerce_process_order_error', array (
						'order_id' => $order_id, 
						'result' => $result, 
						'method' => __METHOD__, 
						'line' => __LINE__ 
				) );
				
				$order->update_status( 'wc-failed' ); // Payment failed,
				                                      // set status to
				                                      // failed.
				$order->add_order_note( sprintf( __( 'Payment for order failed. Reason: %s', 'braintree' ), $result->message ) );
				
				return $this->order_error();
			}
		} catch( Braintree\Exception $e ) {
			wc_add_notice( sprintf( __( 'There was an error processing your payment. Reason: %s' ), bfwc_get_error_message( $e ) ), 'error' );
			do_action( 'braintree_woocommerce_process_order_exception', array (
					'attribs' => $attribs, 
					'exception' => $e, 
					'order_id' => $order_id, 
					'method' => __METHOD__, 
					'line' => __LINE__ 
			) );
			$order->update_status( 'wc-failed' );
			
			return $this->order_error();
		}
	}

	/**
	 * Process the WooCommerce Subscription.
	 *
	 * @param WC_Order $order_id        	
	 * @return string[]
	 */
	public function process_subscription( $order )
	{
		global $bfwc_order_total;
		
		$subscriptions = wcs_get_subscriptions_for_order( $order );
		
		// There may be Braintree Subscriptions in the order. If so, process them.
		foreach ( $subscriptions as $subscription ) {
			if ( bwcs_is_braintree_subscription( $subscription ) ) {
				$result = $this->process_wcs_braintree_subscription( $subscription, $order );
				
				if ( is_wp_error( $result ) ) {
					wc_add_notice( $result->get_error_message(), 'error' );
					return $this->order_error();
				}
			}
		}
		if ( $order->get_total() > 0 ) {
			
			// Add subscription meta method so it can be saved.
			add_action( "braintree_wc_{$this->id}_save_wc_order_meta", array (
					$this, 
					'save_wc_subscription_meta' 
			), 10, 2 );
			// add methods that are needed for processing a WC subscription
			add_filter( "braintree_woocommerce_{$this->id}_order_attributes", __CLASS__ . '::add_subscription_id', 10, 2 );
			
			return $this->process_order( $order );
		} else {
			// reset the order's original value since it might have been manipulated.
			$order->set_total( $bfwc_order_total );
			
			// ensure the payment method token is set.
			self::$payment_method_token = self::$payment_method_token ? self::$payment_method_token : self::get_request_param( static::$token_id );
			$this->save_synchronized_subscription_meta( $order ); // Save
			                                                      // metadata
			                                                      // for
			                                                      // synchronized
			                                                      // subscription.
			WC()->cart->empty_cart();
			$order->payment_complete();
			
			return $this->order_success( $order ); // There is no order amount to
				                                       // process.
		}
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	public function process_braintree_subscription( $order )
	{
		global $bfwc_order_total;
		$user_id = wp_get_current_user()->ID;
		
		$subscriptions = bfwcs_get_subscriptions_for_order( $order );
		
		foreach ( $subscriptions as $subscription ) {
			
			// subscription already created in Braintree so continue.
			if ( $subscription->is_created() ) {
				// subscription was already created, so remove the subscription price from order.
				bfwcs_calculate_order_total( $order, $subscription );
				continue;
			}
			
			$attribs = array (
					'id' => $subscription->id, 
					'planId' => $subscription->braintree_plan, 
					'price' => $subscription->get_total(), 
					'merchantAccountId' => $subscription->merchant_account_id 
			);
			self::add_order_payment_method( $attribs );
			
			if ( $subscription->never_expires() ) {
				$attribs [ 'neverExpires' ] = true;
			} else {
				$attribs [ 'numberOfBillingCycles' ] = $subscription->subscription_length;
			}
			
			/* calculate the start date etc. */
			if ( $subscription->has_trial() ) {
				$attribs [ 'trialDuration' ] = $subscription->get_trial_length();
				$attribs [ 'trialDurationUnit' ] = $subscription->get_trial_period();
				$attribs [ 'trialPeriod' ] = true;
			} else {
				$attribs [ 'firstBillingDate' ] = $subscription->get_date( 'start' );
			}
			
			if ( $subscription->has_descriptors() ) {
				self::set_subscription_descriptors( $attribs, $subscription );
			}
			
			try {
				$result = Braintree_Subscription::create( apply_filters( "bfwc_{$this->id}_subscription_attributes", $attribs ) );
				
				if ( $result->success ) {
					
					$subscription->set_created( true );
					
					$subscription->update_payment_method_title( braintree_get_payment_title_from_token( $user_id, $result->subscription->paymentMethodToken ) );
					$subscription->update_payment_method_token( $result->subscription->paymentMethodToken );
					$subscription->add_order_note( __( 'Subscription created in Braintree.', 'braintree' ) );
					$subscription->update_status( 'active' );
					
					// subscription was created in Braintree, so remove the total from the order to prevent double charges.
					bfwcs_calculate_order_total( $order, $subscription );
				
				} else {
					$subscription->set_created( false );
					$subscription->update_status( 'failed' );
					$order->update_status( 'failed' );
					$subscription->add_order_note( sprintf( __( 'Subscription creation in Braintree failed. Reason: %s.', 'braintree' ), $result->message ) );
					wc_add_notice( sprintf( __( 'Error processing checkout. Reason: %s', 'braintree' ), $result->message ), 'error' );
					
					// set the original order amount.
					$order->set_total( $bfwc_order_total );
					return $this->order_error();
				}
			} catch( \Braintree\Exception $e ) {
				$subscription->set_created( false );
				$subscription->update_status( 'failed' );
				$order->update_status( 'failed' );
				$subscription->add_order_note( sprintf( __( 'Subscription creation in Braintree failed. Exception: %s.', 'braintree' ), get_class( $e ) ) );
				
				// set the original order amount.
				$order->set_total( $bfwc_order_total );
				return $this->order_error();
			}
		}
		
		// loop has ended. Process order.
		if ( $order->get_total() > 0 ) {
			// there are additional fees that need to be charged, like a signup fee, etc.
			return $this->process_order( $order );
		} else {
			// set the order's total since it may have been manipulated.
			$order->set_total( $bfwc_order_total );
			$order->payment_complete();
			update_post_meta( $order->id, '_payment_method_title', braintree_get_payment_title_from_token( $order->customer_user, $attribs [ 'paymentMethodToken' ] ) );
			update_post_meta( $order->id, '_payment_method_token', $attribs [ 'paymentMethodToken' ] );
			WC()->cart->empty_cart();
			return $this->order_success( $order );
		}
	}

	/**
	 * Process the WC Braintree Subscription.
	 * If any errors are encountered, WP_Error object is returned.
	 *
	 * @param WC_Subscription $subscription        	
	 * @param WC_Order $order        	
	 * @return mixed bool|WP_Error
	 */
	public function process_wcs_braintree_subscription( $subscription, $order )
	{
		// If not a Braintree Subscription or already created in Braintree, continue.
		if ( ! bwcs_is_braintree_subscription( $subscription ) || $subscription->created_in_braintree ) {
			return;
		}
		
		$product = bwcs_get_product_from_subscription( $subscription ); // There should only be one item in each subscription. There can be multiple quantities of the same product.
		
		if ( ! $plan_id = bwcs_get_plan_from_product( $product ) ) {
			return new WP_Error( 'subscription-error', sprintf( __( 'Product %s does not have a subscription plan configured for currency %s.', 'braintree' ), $product->get_title(), get_woocommerce_currency() ) );
		}
		
		$attribs = array (
				'id' => $subscription->id, 
				'planId' => $plan_id, 
				'price' => $subscription->get_total() 
		);
		self::add_order_payment_method( $attribs );
		self::add_merchant_account( $attribs );
		self::add_billing_date( $attribs, $subscription );
		self::add_billing_cycles( $attribs, $subscription );
		
		// let plugins add additional attributes.
		$attribs = apply_filters( "braintree_wcs_{$this->id}_braintree_subscription_attribs", $attribs, $subscription, $this );
		
		try {
			$result = Braintree_Subscription::create( $attribs );
			if ( $result->success ) {
				
				$this->save_wc_braintree_subscription_meta( $subscription, $result->subscription );
				
				do_action( 'braintree_wc_braintree_subscription_success', $result );
				
				// Made it this far so now remove this subscription's total from the order. This will prevent double charges.
				bwcs_calculate_order_total( $order, $subscription );
			} else {
				
				$order->add_order_note( sprintf( __( 'Error processing subscription %s. Reason: %s', 'braintree' ), $subscription->id, $result->message ) );
				
				do_action( 'braintree_wc_braintree_subscription_error', $result );
				
				// Exit method so errors can be displayed on checkout page.
				return new WP_Error( 'subscription-error', sprintf( __( 'There was an error processing your subscription. Reason: %s', 'braintree' ), $result->message ) );
			}
		} catch( \Braintree\Exception $e ) {
			
			do_action( 'braintree_wc_braintree_subscription_exception', array (
					'method' => __METHOD__, 
					'line' => __LINE__, 
					'exception' => $e, 
					'attribs' => $attribs 
			) );
			
			// exit method.
			return new WP_Error( 'subscription-error', __( 'There was an error processing your subscription.', 'braintree' ) );
		}
		
		return true;
	}

	public function process_refund( $order_id, $amount = null, $reason = '' )
	{
		$order = wc_get_order( $order_id );
		
		if ( ! bwc_can_refund_order( $order ) ) {
			return new WP_Error( 'refund-error', sprintf( __( 'Order %s does not contain a transaction Id.', 'braintree' ), $order_id ) );
		}
		
		$id = $order->get_transaction_id();
		
		try {
			$result = Braintree_Transaction::refund( $id, $amount );
			if ( $result->success ) {
				$order->add_order_note( sprintf( __( 'Order was successfully refunded in the amount of %s%s.', 'braintree' ), get_woocommerce_currency_symbol( $order->order_currency ), $amount ) );
				return true;
			} else {
				return new WP_Error( 'refund-error', apply_filters( 'braintree_woocommerce_refund_error', sprintf( __( 'There was an error refunding order %s. Reason: %s', 'braintree' ), $order_id, $result->message ), $result ) );
			}
		} catch( Braintree\Exception\NotFound $e ) {
			return new WP_Error( 'refund-error', sprintf( __( 'Transaction %s was not found in your %s Braintree environment.', 'braintree' ), $id, bt_manager()->get_environment() ) );
		} catch( Braintree\Exception $e ) {
			return new WP_Error( 'refund-error', sprintf( __( 'There was an exception thrown while refunding transaction %s.', 'braintree' ), $id ) );
		}
	}

	/**
	 * Save a payment method in Braintree for the user.
	 * This method is only to be called during order processing.
	 *
	 * @param WC_Order $order        	
	 * @return true|WP_Error
	 */
	public function save_payment_method( $order )
	{
		$nonce = self::get_request_param( static::$nonce_id );
		$user_id = wp_get_current_user()->ID;
		
		try {
			$result = Braintree_PaymentMethod::create( array (
					'paymentMethodNonce' => $nonce, 
					'customerId' => braintree_get_customer_id( $user_id ), 
					'billingAddress' => array (
							'firstName' => $order->billing_first_name, 
							'lastName' => $order->billing_last_name, 
							'postalCode' => $order->billing_postcode, 
							'streetAddress' => $order->billing_address_1, 
							'region' => $order->billing_state, 
							'countryCodeAlpha2' => $order->billing_country 
					) 
			) );
			if ( $result->success ) {
				
				self::$payment_method_token = $result->paymentMethod->token;
				$response = true;
				
				braintree_save_user_payment_method( wp_get_current_user()->ID, $result->paymentMethod );
			} else {
				$response = new WP_Error( 'payment-error', sprintf( __( 'Error saving payment method. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) ), 'error' );
				do_action( 'braintree_woocommerce_process_order_error', array (
						'order_id' => $order->id, 
						'result' => $result, 
						'method' => __METHOD__, 
						'line' => __LINE__ 
				) );
			}
		} catch( \Braintree\Exception $e ) {
			do_action( 'braintree_error_wc_subscription_save_method' );
			$response = new WP_Error( 'payment-error', sprintf( __( 'There was an error saving your payment method. Reason: %s', 'braintree' ), bfwc_get_error_message( $e ) ) );
		}
		return $response;
	}

	public function order_success( $order )
	{
		return array (
				'result' => 'success', 
				'redirect' => $order->get_checkout_order_received_url() 
		);
	}

	public function order_error()
	{
		return array (
				'result' => 'failure', 
				'redirect' => '' 
		);
	}

	/**
	 * Does the WC order contain WC subscriptions? If yes, return true.
	 * This method is plugin safe meaning
	 * it will not execute the wcs_order_contains_subscription method if it
	 * doesn't exist.
	 *
	 * @param int $order_id        	
	 * @return boolean
	 */
	public function order_has_subscription( $order_id )
	{
		if ( ! function_exists( 'wcs_order_contains_subscription' ) ) {
			return false;
		}
		return wcs_order_contains_subscription( $order_id );
	}

	/**
	 * Return true if the user has chose to use a saved payment method or the user selected to
	 * save the payment method during checkout.
	 *
	 * @return bool
	 */
	public static function use_saved_method()
	{
		$method = self::get_request_param( static::$token_id );
		return ! empty( $method ) || ! empty( self::$payment_method_token );
	}

	/**
	 * Return the given param from the $_POST.
	 * If the param does not exists, return an empty string.
	 *
	 * @param int $param        	
	 * @return string|mixed
	 */
	public static function get_request_param( $param )
	{
		return isset( $_POST [ $param ] ) ? $_POST [ $param ] : '';
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Order $order        	
	 */
	public static function add_customer( &$attribs, $order )
	{
		$customer_id = bt_manager()->get_customer_id( $order->customer_user );
		if ( $customer_id ) {
			$attribs [ 'customerId' ] = $customer_id;
		}
		$attribs [ 'customer' ] = array (
				'email' => $order->billing_email, 
				'firstName' => $order->billing_first_name, 
				'lastName' => $order->billing_last_name, 
				'phone' => $order->billing_phone 
		);
		return $attribs;
	}

	/**
	 * Add a payment method nonce or token to the order attributes array.
	 *
	 * @param array $attribs        	
	 */
	public static function add_order_payment_method( &$attribs )
	{
		if ( self::use_saved_method() ) {
			$attribs [ 'paymentMethodToken' ] = self::$payment_method_token ? self::$payment_method_token : self::get_request_param( static::$token_id );
		} else {
			$attribs [ 'paymentMethodNonce' ] = self::get_request_param( static::$nonce_id );
		}
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Order $order        	
	 */
	public static function add_order_id( &$attribs, $order )
	{
		$order_prefix = bt_manager()->get_option( 'order_prefix' );
		$attribs [ 'orderId' ] = $order_prefix . $order->get_order_number();
		
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Order $order        	
	 */
	public static function add_subscription_id( $attribs, $order )
	{
		$order_prefix = bt_manager()->get_option( 'woocommerce_subscriptions_prefix' );
		$attribs [ 'orderId' ] = $order_prefix . $order->get_order_number();
		
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 */
	public static function add_merchant_account( &$attribs )
	{
		$merchant_account = bwc_get_merchant_account();
		
		if ( ! empty( $merchant_account ) ) {
			
			$attribs [ 'merchantAccountId' ] = $merchant_account;
		}
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Order $order        	
	 */
	public static function add_billing_address( &$attribs, $order )
	{
		$attribs [ 'billing' ] = array (
				'firstName' => $order->billing_first_name, 
				'lastName' => $order->billing_last_name, 
				'locality' => $order->billing_city, 
				'postalCode' => $order->billing_postcode, 
				'region' => $order->billing_state, 
				'streetAddress' => $order->billing_address_1, 
				'extendedAddress' => $order->billing_address_2, 
				'countryCodeAlpha2' => $order->billing_country 
		);
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Order $order        	
	 */
	public static function add_shipping_address( &$attribs, $order )
	{
		$attribs [ 'shipping' ] = array (
				'firstName' => $order->shipping_first_name, 
				'lastName' => $order->shipping_last_name, 
				'locality' => $order->shipping_city, 
				'postalCode' => $order->shipping_postcode, 
				'region' => $order->shipping_state, 
				'streetAddress' => $order->shipping_address_1, 
				'extendedAddress' => $order->shipping_address_2, 
				'countryCodeAlpha2' => $order->shipping_country 
		);
		return $attribs;
	}

	public static function add_options( &$attribs )
	{
		$attribs [ 'options' ] = array (
				'submitForSettlement' => ! bt_manager()->is_active( 'authorize_transaction' ) 
		);
		if ( bwc_is_3ds_active() ) {
			if ( ! self::use_saved_method() ) { // threeDSecure options attribute
			                                    // can only be added if the
			                                    // payment method nonce is
			                                    // present.
				$attribs [ 'options' ] [ 'threeDSecure' ] = array (
						'required' => true 
				);
			}
		}
		return $attribs;
	}

	public static function add_partner_code( &$attribs )
	{
		$attribs [ 'channel' ] = BT_Manager()->get_partner_code();
		return $attribs;
	}

	public static function add_descriptors( &$attribs )
	{
		if ( bwc_is_descriptors_enabled() ) {
			$attribs [ 'descriptor' ] = array (
					'name' => bt_manager()->get_option( 'dynamic_descriptor_name' ), 
					'phone' => bt_manager()->get_option( 'dynamic_descriptor_phone' ), 
					'url' => bt_manager()->get_option( 'dynamic_descriptor_url' ) 
			);
		}
		return $attribs;
	}

	public static function add_device_data( &$attribs )
	{
		if ( bwc_is_advanced_fraud_tools() ) {
			$attribs [ 'deviceData' ] = self::get_request_param( static::$device_data_id );
		}
		return $attribs;
	}

	/**
	 * Adds the Subscription billing date.
	 *
	 * @param array $attribs        	
	 * @param WC_Subscription $subscription        	
	 */
	public static function add_billing_date( &$attribs, $subscription )
	{
		$start_time_in_utc = bwcs_get_start_date_in_utc( $subscription );
		$current_time_in_utc = DateTime::createFromFormat( 'Y-m-d H:i:s', current_time( 'mysql' ) );
		$current_time_in_utc = new DateTime( $current_time_in_utc->format( 'Y-m-d H:i:s' ), new DateTimeZone( 'UTC' ) );
		
		// set the hours, minutes, seconds equal to the values of start_time so that any difference calulcations will be in whole days.
		$current_time_in_utc->setTime( $start_time_in_utc->format( 'H' ), $start_time_in_utc->format( 'i' ), $start_time_in_utc->format( 's' ) );
		
		$has_trial = ( bool ) $subscription->get_time( 'trial_end' );
		$attribs [ 'trialPeriod' ] = $has_trial;
		
		if ( $has_trial ) {
			$difference = $start_time_in_utc->diff( $current_time_in_utc );
			$attribs [ 'trialDurationUnit' ] = 'day';
			$attribs [ 'trialDuration' ] = ( int ) $difference->days;
		} else {
			$attribs [ 'firstBillingDate' ] = $start_time_in_utc;
		}
		
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param WC_Subscription $subscription        	
	 */
	public static function add_subscription_options( &$attribs, $subscription )
	{
		$has_trial = ( bool ) $subscription->get_time( 'trial_end' );
		
		$attribs [ 'options' ] = array (
				'startImmediately' => ! ( bool ) $subscription->contains_synced_subscription && ! $has_trial 
		);
		
		return $attribs;
	}

	/**
	 * Add the billing cycles to the Braintree Subscription.
	 *
	 * @param array $attribs        	
	 * @param WC_Subscription $subscription        	
	 */
	public static function add_billing_cycles( &$attribs, $subscription )
	{
		// If there is no end date then the subscription never expires.
		if ( $subscription->get_date( 'end' ) === 0 ) {
			$attribs [ 'neverExpires' ] = true;
		} else {
			$product = bwcs_get_product_from_subscription( $subscription );
			$attribs [ 'numberOfBillingCycles' ] = WC_Subscriptions_Product::get_length( $product );
		}
		return $attribs;
	}

	/**
	 *
	 * @param array $attribs        	
	 * @param Braintree_Gateway_WC_Subscription $subscription        	
	 */
	public static function set_subscription_descriptors( &$attribs, $subscription )
	{
		if ( $name = $subscription->get_descriptor( 'name' ) ) {
			$attribs [ 'descriptor' ] [ 'name' ] = $name;
		}
		if ( $phone = $subscription->get_descriptor( 'phone' ) ) {
			$attribs [ 'descriptor' ] [ 'phone' ] = $phone;
		}
		if ( $url = $subscription->get_descriptor( 'url' ) ) {
			$attribs [ 'descriptor' ] [ 'url' ] = $url;
		}
		return $attribs;
	}

	public function payment_fields()
	{
		if ( bwc_is_checkout() || bwcs_is_change_payment_method() || bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			
			bwc_get_template( 'checkout/braintree-checkout.php', array (
					'gateway' => $this 
			) );
		} elseif ( is_add_payment_method_page() ) {
			
			if ( bwc_is_custom_form() ) {
				bwc_get_template( 'custom-form.php', array (
						'has_methods' => false, 
						'gateway' => $this 
				) );
			} else {
				
				bwc_get_template( 'dropin-form.php', array (
						'has_methods' => false, 
						'gateway' => $this 
				) );
			}
		}
	}

	public function set_supports()
	{
		$this->supports = array (
				'subscriptions', 
				'products', 
				'add_payment_method', 
				'subscription_cancellation', 
				'multiple_subscriptions', 
				'subscription_amount_changes', 
				'subscription_date_changes', 
				'default_credit_card_form', 
				'refunds', 
				'pre-orders', 
				'subscription_payment_method_change_admin', 
				'subscription_reactivation', 
				'subscription_suspension', 
				'subscription_payment_method_change_customer', 
				'bfwcs_change_payment_method', 
				'bfwc_subscriptions' 
		);
	}

	/**
	 * Save the order meta for the transaction.
	 *
	 * @param int $order_id        	
	 * @param Braintree_Transaction $transaction        	
	 */
	public function save_order_meta( $order_id, $transaction )
	{
		update_post_meta( $order_id, '_braintree_version', bt_manager()->version );
		update_post_meta( $order_id, '_merchant_account_id', $transaction->merchantAccountId );
		update_post_meta( $order_id, '_payment_method_token', braintree_get_payment_token_from_transaction( $transaction ) );
		update_post_meta( $order_id, '_payment_method_title', braintree_get_payment_method_title_from_transaction( $transaction ) );
	}

	/**
	 * Save the subscription order meta for the transaction.
	 *
	 * @param int $order_id        	
	 * @param Braintree_Transaction $transaction        	
	 */
	public function save_wc_subscription_meta( $order_id, $transaction )
	{
		if ( ! function_exists( 'wcs_get_subscriptions_for_order' ) ) {
			return;
		}
		$subscriptions = wcs_get_subscriptions_for_order( $order_id );
		foreach ( $subscriptions as $subscription ) { // save the subscription
		                                              // metadata.
			if ( ! bwcs_is_braintree_subscription( $subscription ) ) { // Only save non Braintree Subscriptions meta.
				$this->save_order_meta( $subscription->id, $transaction );
				
				update_post_meta( $subscription->id, '_subscription_type', 'woocommerce' );
			}
		}
	}

	/**
	 * Save the subscription metadata for the synchronized subscription.
	 * This method saves the metadata to the WC_Order and then to each
	 * WC_Subscription
	 * that is contained within the order.
	 *
	 * @param WC_Order $order        	
	 */
	public function save_synchronized_subscription_meta( $order )
	{
		$payment_method_title = braintree_get_payment_method_title_from_array( braintree_get_payment_method_from_token( wp_get_current_user()->ID, self::$payment_method_token ) );
		
		// Save the metadata for the order.
		update_post_meta( $order->id, '_braintree_version', bt_manager()->version );
		update_post_meta( $order->id, '_merchant_account_id', bwc_get_merchant_account() );
		update_post_meta( $order->id, '_payment_method_token', self::$payment_method_token );
		update_post_meta( $order->id, '_payment_method_title', $payment_method_title );
		if ( $order instanceof WC_Subscription ) {
			update_post_meta( $order->id, '_subscription_type', 'woocommerce' );
		}
		
		$subscriptions = wcs_get_subscriptions_for_order( $order );
		
		// Save the metadata for the subscriptions.
		foreach ( $subscriptions as $subscription ) {
			if ( ! bwcs_is_braintree_subscription( $subscription ) ) { // only save non braintree subscriptions.
				$this->save_synchronized_subscription_meta( $subscription );
			}
		}
	}

	/**
	 * Save the subscription metadata for the Braintree Subscription.
	 *
	 * @param WC_Subscription $subscription        	
	 * @param Braintree_Subscription $bt_subscription        	
	 */
	public function save_wc_braintree_subscription_meta( $subscription, $bt_subscription )
	{
		$user_id = wp_get_current_user()->ID;
		$token = $bt_subscription->paymentMethodToken;
		$payment_method = braintree_get_payment_method_from_token( $user_id, $token );
		$payment_method_title = braintree_get_payment_method_title_from_array( $payment_method );
		$post_id = $subscription->id;
		
		update_post_meta( $post_id, '_payment_method', $this->id );
		update_post_meta( $post_id, '_braintree_version', bt_manager()->version );
		update_post_meta( $post_id, '_created_in_braintree', true );
		update_post_meta( $post_id, '_merchant_account_id', $bt_subscription->merchantAccountId );
		update_post_meta( $post_id, '_payment_method_token', $token );
		update_post_meta( $post_id, '_payment_method_title', $payment_method_title );
		update_post_meta( $post_id, '_braintree_plan', $bt_subscription->planId );
		update_post_meta( $post_id, '_subscription_type', 'braintree' );
	}

	/**
	 * If the setting for updating the order status is enabled, allow the plugin
	 * order status to be set.
	 * This will override the status set by WooCommerce.
	 *
	 * @param string $status        	
	 * @param int $id        	
	 * @return string
	 */
	public static function maybe_update_order_status( $status, $id )
	{
		$order = wc_get_order( $id );
		if ( $order->payment_method === static::ID ) {
			$order_status = bt_manager()->get_option( 'order_status' );
			
			if ( $order_status !== 'default' ) {
				$status = $order_status;
			}
		}
		return $status;
	}

	public function woocommerce_gateway_icon( $icon, $id )
	{
		if ( $id === $this->id ) {
			if ( bwc_payment_icons_outside() ) {
				ob_start();
				bwc_get_template( 'checkout/payment-method-icons.php' );
				$icon = ob_get_clean();
			}
		}
		return $icon;
	}

	public function woocommerce_gateway_title( $title, $id )
	{
		if ( $id === $this->id ) {
			if ( bt_manager()->get_environment() === 'sandbox' ) {
				$title = sprintf( '%s ( %s )', $title, __( 'Sandbox', 'braintree' ) );
			}
		}
		return $title;
	}

	/**
	 * Populate an array of saved payment methods for the user.
	 * This array of payment methods is displayed
	 * on the payment-methods page.
	 *
	 * @param array $saved_methods        	
	 * @param int $user_id        	
	 */
	public function saved_payment_method_list( $saved_methods, $user_id )
	{
		$methods = bwc_get_user_payment_methods( $user_id );
		
		return bwc_saved_payment_methods_list( $saved_methods, $methods, $this->id );
	}

	/**
	 * Check to see if this is a request to delete a payment method.
	 * If true, then delete the payment method.
	 * Do not allow a user to delete a payment method if there is a subscription
	 * associated with it.
	 */
	public static function maybe_delete_payment_method()
	{
		$nonce = isset( $_GET [ '_wpnonce' ] ) ? $_GET [ '_wpnonce' ] : '';
		
		if ( isset( $_GET [ static::ID . '_delete_method' ] ) && wp_verify_nonce( $nonce, 'delete-payment-method' ) ) {
			
			$user_id = wp_get_current_user()->ID;
			$token = $_GET [ static::ID . '_delete_method' ];
			
			// Allow other functionality to prevent the delete from occuring.
			$can_delete = apply_filters( 'braintree_wc_can_delete_payment_method', new WP_Error(), $token );
			
			if ( $can_delete->get_error_message() ) {
				wc_add_notice( $can_delete->get_error_message(), 'error' );
				return;
			}
			
			try {
				$result = PaymentMethod::delete( $token );
				
				if ( $result->success ) {
					
					braintree_delete_user_payment_method( $user_id, $token );
					
					wc_add_notice( __( 'Your payment method has been deleted.', 'braintree' ), 'success' );
					do_action( 'braintree_wc_delete_payment_method_success', $result, $token );
				} else {
					wc_add_notice( sprintf( __( 'Your payment method could not be deleted. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) ), 'error' );
					
					do_action( 'braintree_wc_delete_payment_method_error', $result );
				}
			} catch( \Braintree\Exception $e ) {
				do_action( 'braintree_wc_delete_payment_method_exception', array (
						'user_id' => $user_id, 
						'exception' => $e, 
						'token' => $token, 
						'method' => __METHOD__, 
						'line' => __LINE__ 
				) );
				wc_add_notice( sprintf( __( 'Your payment method could not be deleted at this time. Reason: %s', 'braintree' ), bfwc_get_error_message( $e ) ), 'error' );
			}
		}
	}

	/**
	 * Add a payment method to the user's account after vaulting it in Braintree.
	 */
	public function add_payment_method( $attribs = array() )
	{
		$nonce = self::get_request_param( static::$nonce_id );
		$user_id = wp_get_current_user()->ID;
		
		try {
			$default_attribs = array (
					'customerId' => braintree_get_customer_id( $user_id ), 
					'paymentMethodNonce' => $nonce, 
					'options' => array (
							'failOnDuplicatePaymentMethod' => bwc_fail_on_duplicate(), 
							'makeDefault' => true, 
							'verifyCard' => true 
					) 
			);
			$attribs = wp_parse_args( $attribs, $default_attribs );
			
			if ( bwc_is_advanced_fraud_tools() ) {
				$attribs [ 'deviceData' ] = self::get_request_param( static::$device_data_id );
			}
			$result = Braintree_PaymentMethod::create( $attribs );
			
			if ( $result->success ) {
				
				braintree_save_user_payment_method( $user_id, $result->paymentMethod );
				self::$payment_method_token = $result->paymentMethod->token;
				do_action( 'braintree_wc_add_payment_method_success', $result );
				return array (
						'result' => 'success', 
						'redirect' => wc_get_endpoint_url( 'payment-methods' ) 
				);
			} else {
				do_action( 'braintree_wc_add_payment_method_error', $result );
				wc_add_notice( sprintf( __( 'There was an error saving your payment method. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) ), 'error' );
				return array (
						'result' => 'failure' 
				);
			}
		} catch( \Braintree\Exception $e ) {
			do_action( 'braintree_wc_add_payment_method_exception', array (
					'exception' => $e, 
					'method' => __METHOD__, 
					'line' => __LINE__, 
					'nonce' => $nonce 
			) );
			wc_add_notice( sprintf( __( 'There was an error saving your payment method. Reason: %s', 'braintree' ), bfwc_get_error_message( $e ) ), 'error' );
			return array (
					'result' => 'failure' 
			);
		}
	}

	/**
	 * Update the payment method for the subscription.
	 * If the subscription is from another gateway,
	 * then simply update the payment method token. Even if the product is configured as a Braintree Subscription,
	 * it is better to let WC Subscriptions continue to handle this subscription, rather then create a new Braintree Subscription.
	 *
	 * @param WC_Subscription $subscription        	
	 * @param string $old_payment_method        	
	 */
	public function payment_method_updated_to( $subscription, $old_payment_method )
	{
		if ( ! self::use_saved_method() ) {
			// save the paymetn method
			$result = $this->add_payment_method();
			if ( $result [ 'result' ] !== 'success' ) {
				update_post_meta( $subscription->id, '_new_payment_method', $old_payment_method );
				update_post_meta( $subscription->id, '_payment_method', $old_payment_method );
				return;
			}
			$token = self::$payment_method_token;
		} else {
			$token = self::get_request_param( static::$token_id );
		}
		
		$payment_method_title = braintree_get_payment_method_title_from_array( braintree_get_payment_method_from_token( wp_get_current_user()->ID, $token ) );
		
		// subscription must be a Braintree subscription and it must simply be a payment method change and not a payment gateway change.
		if ( bwcs_is_braintree_subscription( $subscription ) && $subscription->payment_method === $old_payment_method ) {
			try {
				$result = Braintree_Subscription::update( $subscription->id, array (
						'paymentMethodToken' => $token 
				) );
				if ( $result->success ) {
					update_post_meta( $subscription->id, '_payment_method_token', $token );
					update_post_meta( $subscription->id, '_payment_method_title', $payment_method_title );
				} else {
					wc_add_notice( sprintf( __( 'Error changing payment methods for subscription. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) ), 'error' );
					update_post_meta( $subscription->id, '_new_payment_method', $old_payment_method );
					update_post_meta( $subscription->id, '_payment_method', $old_payment_method );
				}
			} catch( \Braintree\Exception $e ) {
				wc_add_notice( __( 'Error changing payment method for subscription.', 'braintree' ), 'error' );
				update_post_meta( $subscription->id, '_new_payment_method', $old_payment_method );
				update_post_meta( $subscription->id, '_payment_method', $old_payment_method );
			}
		} else {
			update_post_meta( $subscription->id, '_payment_method_token', $token );
			update_post_meta( $subscription->id, '_payment_method_title', $payment_method_title );
		}
	}

	/**
	 * Update the Braintree subscription's payment method.
	 *
	 * @param Braintree_Gateway_WC_Subscription $subscription        	
	 * @return mixed bool|WP_Error
	 */
	public function change_subscription_payment_method( $subscription )
	{
		$user_id = wp_get_current_user()->ID;
		
		if ( ! self::use_saved_method() ) {
			
			// save the payment method
			$result = $this->add_payment_method();
			if ( $result [ 'result' ] === 'failure' ) {
				return new WP_Error();
			}
			$token = self::$payment_method_token;
		} else {
			$token = self::get_request_param( static::$token_id );
		}
		$result = true;
		try {
			$result = Braintree_Subscription::update( $subscription->id, array (
					'paymentMethodToken' => $token 
			) );
			
			if ( $result->success ) {
				update_post_meta( $subscription->id, '_payment_method_title', braintree_get_payment_title_from_token( $user_id, $token ) );
				update_post_meta( $subscription->id, '_payment_method_token', $result->subscription->paymentMethodToken );
				bt_manager()->success( sprintf( __( 'Payment method for Braintree subscription %s updated to %s.', 'braintree' ), $subscription->id, $subscription->get_payment_method_to_display() ) );
			} else {
				bt_manager()->error( sprintf( __( 'Error changing payment method for Braintree subscription %s. Reason: %s', 'braintree' ), $subscription->id, $result->message ) );
				$result = new WP_Error( 'subscription-error', __( 'Error changing payment method for subscription. Reason: %s', 'braintree' ), $result->message );
			}
		} catch( \Braintree\Exception $e ) {
			$message = sprintf( __( 'Error changing payment method for Braintree subscription %s. Exception: %s.', 'braintree' ), $subscription->id, get_class( $e ) );
			$subscription->add_order_note( $message );
			bt_manager()->error( $message );
			$result = new WP_Error( 'subscription-error', __( 'There was an error changing your payment method. If the issue continues please contact us.', 'braintree' ) );
		}
		return $result;
	}

	public static function output_new_methods_text()
	{
		if ( is_checkout() || bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			echo '<div class="payment-method-buttons"><a href="#" class="cancel-saved">' . __( 'Cancel', 'braintree' ) . '</a></div>';
		}
	}

	public static function output_saved_methods_text()
	{
		if ( is_checkout() || bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			echo '<div class="payment-method-buttons"><a href="#" class="saved-methods"> ' . __( 'Saved Cards', 'braintree' ) . '</a></div>';
		}
	}

	/**
	 * Save the payment method data from the paid for order to the subscription metadata so future renewal orders won't fail.
	 *
	 * @param WC_Subscription $subscription        	
	 * @param WC_Order $order        	
	 */
	public static function update_failing_payment_method( $subscription, $order )
	{
		// only update the subscription's payment method if the order was paid for using a saved payment method.
		if ( $order->payment_method_token ) {
			update_post_meta( $subscription->id, '_payment_method_token', $order->payment_method_token );
			update_post_meta( $subscription->id, '_payment_method_title', $order->payment_method_title );
		}
	}

	/**
	 *
	 * @param int $subscription_id        	
	 *
	 * @return mixed bool|WP_Error
	 */
	public function pay_for_braintree_subscription( $subscription_id )
	{
		$subscription = bfwcs_get_subscription( $subscription_id );
		
		if ( ! self::use_saved_method() ) {
			$result = $this->add_payment_method();
			if ( $result [ 'result' ] === 'failure' ) {
				return new WP_Error();
			}
		}
		
		$attribs = array (
				'id' => $subscription->id, 
				'price' => $subscription->get_total(), 
				'planId' => $subscription->braintree_plan, 
				'merchantAccountId' => $subscription->merchant_account_id 
		);
		self::add_order_payment_method( $attribs );
		if ( $subscription->never_expires() ) {
			$attribs [ 'neverExpires' ] = true;
		} else {
			$attribs [ 'numberOfBillingCycles' ] = $subscription->subscription_length;
		}
		
		/* calculate the start date etc. */
		if ( $subscription->has_trial() ) {
			$attribs [ 'trialDuration' ] = $subscription->get_trial_length();
			$attribs [ 'trialDurationUnit' ] = $subscription->get_trial_period();
			$attribs [ 'trialPeriod' ] = true;
		} else {
			$attribs [ 'firstBillingDate' ] = $subscription->get_date( 'start' );
		}
		
		if ( $subscription->has_descriptors() ) {
			self::set_subscription_descriptors( $attribs, $subscription );
		}
		
		try {
			$result = Braintree_Subscription::create( $attribs );
			if ( $result->success ) {
				$subscription->update_status( 'active' );
				$subscription->add_order_note( __( 'Subscription created successfully in Braintree.', 'braintree' ) );
				$subscription->update_payment_method_title( braintree_get_payment_title_from_token( $user_id, $result->subscription->paymentMethodToken ) );
				$subscription->update_payment_method_token( $result->subscription->paymentMethodToken );
				return true;
			} else {
				$message = sprintf( __( 'Error creating subscription. Reason: %s', 'braintree' ), $result->message );
				return new WP_Error( 'subscription-error', $message );
			}
		} catch( \Braintree\Exception $e ) {
			return new WP_Error( 'subscription-error', sprintf( __( 'There was an error creating your subscription. Please contact us if the issue continues.', 'braintree' ) ) );
		}
	}

	/**
	 * Add customer metadata before a new user is inserted into the database.
	 *
	 * @param array $customer_data        	
	 * @param WP_User $user        	
	 * @param bool $update        	
	 */
	public static function woocommerce_new_customer_data( $customer_data, $user, $update )
	{
		// if called during checkout, then proceed with adding customer data. This will ensure
		// that when a customer is created within Braintree, the firstName, lastName, etc is populated.
		if ( defined( 'WOOCOMMERCE_CHECKOUT' ) && ! $update ) {
			$checkout = WC()->checkout();
			$customer_data [ 'first_name' ] = $checkout->posted [ 'billing_first_name' ];
			$customer_data [ 'last_name' ] = $checkout->posted [ 'billing_last_name' ];
			$customer_data [ 'billing_phone' ] = $checkout->posted [ 'billing_phone' ];
			$customer_data [ 'billing_email' ] = $checkout->posted [ 'billing_email' ];
			$customer_data [ 'billing_company' ] = $checkout->posted [ 'billing_company' ];
		}
		return $customer_data;
	}

	/**
	 * Perform gateway specific validation.
	 *
	 * @param WC_Order $order        	
	 */
	public static function before_order_process( $order )
	{
		do_action( "bfwc_before_process_order_{$order->payment_method}", $order );
	}

	/**
	 * Hooked into the 'bfwc_before_process_order_' + gateway_id action.
	 * If a customer is requesting to save a payment method, then save it here.
	 *
	 * @param WC_Order $order        	
	 */
	public static function maybe_save_payment_method( $order )
	{
		// customer has chosen a saved payment method so exit;
		if ( self::use_saved_method() ) {
			return;
		}
		$save_method = false;
		if ( class_exists( 'WC_Subscriptions_Cart' ) && WC_Subscriptions_Cart::cart_contains_subscription() ) {
			$save_method = true;
		} elseif ( bfwcs_cart_contains_subscriptions() ) {
			$save_method = true;
		} elseif ( ! empty( $_POST [ static::$save_method_name ] ) ) {
			$save_method = true;
		}
		if ( $save_method ) {
			$available_gateways = WC()->payment_gateways()->get_available_payment_gateways();
			$gateway = $available_gateways [ $order->payment_method ];
			
			// add the payment method.
			$result = $gateway->add_payment_method( array (
					'billingAddress' => array (
							'firstName' => $order->billing_first_name, 
							'lastName' => $order->billing_last_name, 
							'postalCode' => $order->billing_postcode, 
							'streetAddress' => $order->billing_address_1, 
							'region' => $order->billing_state, 
							'countryCodeAlpha2' => $order->billing_country 
					) 
			) );
		}
	}

	public function admin_options()
	{
		bfwc_admin_get_template( 'views/admin-gateway-options.php', array (
				'gateway' => $this 
		) );
	}
}
WC_Braintree_Payment_Gateway::init();