<?php
class Braintree_Gateway_WC_Subscriptions_Checkout
{

	public function __construct()
	{
	
	}

	public static function init()
	{
		add_action( 'woocommerce_checkout_order_processed', __CLASS__ . '::process_checkout', 1000, 2 );
		
		add_action( 'woocommerce_review_order_after_order_total', __CLASS__ . '::after_order_total' );
		
		add_action( 'woocommerce_before_pay_action', __CLASS__ . '::pay_order_action' );
	}

	/**
	 *
	 * @param int $order_id        	
	 * @param array $posted_data        	
	 */
	public static function process_checkout( $order_id, $posted_data )
	{
		$order = wc_get_order( $order_id );
		
		$subscriptions = bfwcs_get_subscriptions_for_order( $order_id );
		
		$recurring_carts = WC()->cart->recurring_carts;
		
		foreach ( $subscriptions as $subscription ) {
			if ( $subscription->is_created() ) {
				$session_cart_data = WC()->session->get( 'bfwcs_cart_data', array () );
				// unset the recurring cart associated with this subscription to prevent duplicate processing.
				unset( $recurring_carts [ $session_cart_data [ $subscription->id ] ] );
			} else {
				// delete any existing subscriptions associated with the order that haven't been created in Braintree yet.
				wp_delete_post( $subscription->id );
			}
		}
		
		foreach ( $recurring_carts as $key => $recurring_cart ) {
			self::create_subscription( $order, $recurring_cart, $key );
		}
	}

	/**
	 * Create the subscription object for the order.
	 *
	 * @param WC_Order $order        	
	 * @param WC_Cart $recurring_cart        	
	 * @param string $cart_key        	
	 */
	public static function create_subscription( $order, $recurring_cart, $recurring_cart_key )
	{
		$subscription = bfwcs_create_subscription( array (
				'order_id' => $order->id, 
				'start_date' => $recurring_cart->start_date, 
				'subscription_trial_length' => $recurring_cart->subscription_trial_length, 
				'subscription_trial_period' => $recurring_cart->subscription_trial_period, 
				'trial_end_date' => $recurring_cart->trial_end_date, 
				'next_payment_date' => $recurring_cart->next_payment_date, 
				'first_payment_date' => $recurring_cart->first_payment_date, 
				'end_date' => $recurring_cart->end_date, 
				'braintree_plan' => $recurring_cart->braintree_plan, 
				'subscription_period' => $recurring_cart->subscription_period, 
				'subscription_period_interval' => $recurring_cart->subscription_period_interval, 
				'subscription_length' => $recurring_cart->subscription_length, 
				'subscription_time_zone' => $recurring_cart->subscription_time_zone, 
				'merchant_account_id' => $recurring_cart->merchant_account_id, 
				'descriptors' => $recurring_cart->descriptors, 
				'customer_user' => $order->get_user_id(), 
				'order_note' => $order->customer_note 
		) );
		
		if ( is_wp_error( $subscription ) ) {
			throw new Exception( $subscription->get_error_message() );
		}
		
		// update the billing and shipping addresses.
		$subscription = bfwc_copy_address_from_order( $order, $subscription );
		
		// add the line items
		foreach ( $recurring_cart->get_cart() as $cart_key => $cart_item ) {
			$subscription->add_product( $cart_item [ 'data' ], $cart_item [ 'quantity' ], array (
					'variation' => $cart_item [ 'variation' ], 
					'total' => array (
							'subtotal' => $cart_item [ 'line_subtotal' ], 
							'subtotal_tax' => $cart_item [ 'line_subtotal_tax' ], 
							'total' => $cart_item [ 'line_total' ], 
							'tax' => $cart_item [ 'line_tax' ], 
							'tax_data' => $cart_item [ 'line_tax_data' ] 
					) 
			) );
		}
		$gateways = WC()->payment_gateways()->get_available_payment_gateways();
		
		if ( isset( $gateways [ $order->payment_method ] ) ) {
			$subscription->set_payment_method( $gateways [ $order->payment_method ] );
		}
		
		// add fees
		foreach ( $recurring_cart->get_fees() as $fee_key => $fee ) {
			if ( ! $fee_id = $subscription->add_fee( $fee ) ) {
				throw new Exception( sprintf( __( 'Error %d: Subscription cannot be created.', 'braintree' ), 403 ) );
			}
		}
		
		// add coupons
		foreach ( $recurring_cart->get_coupons() as $code => $coupon ) {
			if ( ! $coupon_id = $subscription->add_coupon( $code, $recurring_cart->get_coupon_discount_amount( $code ), $recurring_cart->get_coupon_discount_tax_amount( $code ) ) ) {
				throw new Exception( sprintf( __( 'Error adding coupons: Subscription cannot be created. Please try again.', 'braintree' ) ) );
			}
		}
		
		foreach ( $recurring_cart->get_taxes() as $tax_rate_id => $amount ) {
			if ( $tax_rate_id && ! $subscription->add_tax( $tax_rate_id, $recurring_cart->get_tax_amount( $tax_rate_id ), $recurring_cart->get_shipping_tax_amount( $tax_rate_id ) ) && apply_filters( 'woocommerce_cart_remove_taxes_zero_rate_id', 'zero-rated' ) !== $tax_rate_id ) {
				throw new Exception( sprintf( __( 'Error adding taxes: Unable to create subscription. Please try again.', 'braintree' ) ) );
			}
		}
		
		$subscription->set_total( $recurring_cart->shipping_total, 'shipping' );
		$subscription->set_total( $recurring_cart->get_cart_discount_total(), 'cart_discount' );
		$subscription->set_total( $recurring_cart->get_cart_discount_tax_total(), 'cart_discount_tax' );
		$subscription->set_total( $recurring_cart->tax_total, 'tax' );
		$subscription->set_total( $recurring_cart->shipping_tax_total, 'shipping_tax' );
		$subscription->set_total( $recurring_cart->total );
		
		// store the subscription Id to cart_key in the WC session. This can be used later.
		$session_cart_data = WC()->session->get( 'bfwcs_cart_data', array () );
		$session_cart_data [ $subscription->id ] = $recurring_cart_key;
		WC()->session->set( 'bfwcs_cart_data', $session_cart_data );
		
		do_action( 'bfwc_subscription_created', $subscription, $order );
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	public static function pay_order_action( $order )
	{
		self::process_checkout( $order->id, array () );
	}

	public static function after_order_total()
	{
		$recurring_carts = WC()->cart->recurring_carts;
		if ( $recurring_carts ) {
			bwc_get_template( 'checkout/review-order.php', array (
					'recurring_carts' => $recurring_carts 
			) );
		}
	}
}
Braintree_Gateway_WC_Subscriptions_Checkout::init();